@@ -11,7 +11,7 @@
 #' @author T. Carruthers
 #' @export 
 plot.OM <-function(x, ...){  
-    OM <- x
+    OM <- updateMSE(x) # update and add missing slots with default values
     out<-runMSE(OM,Hist=T)
     nsim<-OM@nsim
     nyears<-OM@nyears
